@@ -47,11 +47,15 @@ class NumpyEncoder(json.JSONEncoder):
             return float(obj)
         return json.JSONEncoder.default(self, obj)
 
-def toiter(obj):
+def toiter(obj, is_iter=False):
   try:
     iter(obj)
+    if is_iter:
+      return obj, True
     return obj 
   except TypeError:
+    if is_iter:
+      return [ obj ], False
     return [ obj ]
 
 def jsonify(obj, **kwargs):
@@ -733,8 +737,9 @@ class Bbox(object):
     return tmp.astype(int)
 
   def __itruediv__(self, operand):
-    self.minpt /= operand
-    self.maxpt /= operand
+    res = self.__truediv__(operand)
+    self.minpt[:] = res.minpt[:]
+    self.maxpt[:] = res.maxpt[:]
     return self
 
   def __truediv__(self, operand):
